# `OpenTelemetry` Custom Instrumentation Demo
This project provides a small `Spring Boot` backend with a simple `HTTP` interface
to demonstrate custom instrumentation with `OpenTelemetry`.

Involved technologies:
- `Spring Boot` for the backend
  - `Hibernate` / `JPA` for database access
  - `Quartz` for job scheduling 
- `OpenTelemetry` for tracing
- `Zipkin` for trace visualization
- `Docker` for containerization
- `Postgre`SQL`` for the database

The demo shows how a custom `OpenTelemetry` instrumentation can implemented.
The instrumentation is integrated with `Quartz` to connect incoming `HTTP` requests with scheduled jobs.
This example solves a classic problem: How to associate an asynchronous operation (e.g. a scheduled job) with an incoming `HTTP` request

Here's a brief breakdown of the instrumentation classes:
- `QuartzTelemetryInstrumentation` - Instrumentation for `Quartz` jobs which is invoked via a `JobListener` from `Quartz`
- `QuartzJobFacade` - facade to schedule `Quartz` jobs associated with a `Span` from `OpenTelemetry`
- `QuartzInstrumentationJobListener` - `JobListener` implementation to start and end `Spans` for `Quartz` jobs

The demo `HTTP` endpoint `/api/v1/hello-job` triggers a scheduled job which is associated with the incoming `HTTP` request
and also performs additional `SQL` operations to make the resulting trace a bit more interesting.

## How to run the setup?

You need a recent version of `Docker` / `Docker Compose` installed on your machine.
Once `Docker` is up and running, you can start the setup with the following command:

```shell
docker compose up --build
```

Generate some load on the service:

```shell
curl http://localhost:8080/api/v1/hello-job
```

Finally, open [Zipkin UI](http://localhost:9411/zipkin/) to view traces.