package com.dvag.example.otel.quartz.observability

import com.dvag.example.otel.quartz.observability.QuartzInstrumentationJobListener.Companion.TRACE_ID
import io.opentelemetry.api.trace.Span
import io.opentelemetry.sdk.trace.IdGenerator
import org.quartz.*
import org.springframework.stereotype.Service

@Service
class QuartzJobFacade(private val scheduler: Scheduler) {

    /**
     * Adds a job to the scheduler and ensures that
     * the trace context is propagated.
     */
    fun executeNow(clazz: Class<out Job>, jobParameters: Map<String, Any>) {
        // Initialize job data with the current trace ID
        val jobData = JobDataMap(mapOf(Pair(TRACE_ID, getTraceIdFromSpanOrCreateNew())))
        jobData.putAll(jobParameters)
        val jobDetail = JobBuilder.newJob(clazz)
            .usingJobData(JobDataMap(jobData))
            .withIdentity(JobKey.createUniqueName(clazz.simpleName))
            .requestRecovery(true)
            .build()
        val trigger = TriggerBuilder.newTrigger()
            .withIdentity(TriggerKey.createUniqueName(clazz.simpleName))
            .startNow()
            .build()
        scheduler.scheduleJob(jobDetail, trigger)
    }

    private fun getTraceIdFromSpanOrCreateNew(): String {
        if (Span.current().spanContext.isValid) {
            return Span.current().spanContext.traceId
        }
        return IdGenerator.random().generateTraceId()
    }
}